#include <iostream>
#include <cstdio>
#include <string>

using namespace std;

int main(int argc, char ** argv)
{
  if(argc != 2)
  { 
    cerr << "MPO2JPG: Extract stereo JPEG files from MPO file." << endl;
    cerr << argv[0] << ": wrong number of arguments" << endl;
    cerr << "Usage: " << argv[0] << " file.mpo" << endl;
    exit(1);
  }

  string file = argv[1];

  cerr << "Splitting file: " << file << endl;

  size_t pos = file.find_last_of('.');

  string file1, file2;

  if(pos != string::npos)
  {
    file1 = file.substr(0,pos) + "-L.JPG";
    file2 = file.substr(0,pos) + "-R.JPG";
  }
  else
  {
    file1 = file + "-L.JPG";
    file2 = file + "-R.JPG";
  }

  cerr << "New files: " << file1 << " and " << file2 << endl;

  FILE * in, * out1, * out2;

  in = fopen(file.c_str(), "rb");
  if(!in)
  {
    cerr << "Could not open file for reading: " << file << endl;
    exit(1);
  }

  out1 = fopen(file1.c_str(), "wb");
  if(!out1)
  {
    cerr << "Could not open file for writing: " << file1 << endl;
    exit(1);
  }

  out2 = fopen(file2.c_str(), "wb");
  if(!out2)
  {
    cerr << "Could not open file for writing: " << file2 << endl;
    exit(1);
  }

  int c;
  int lb[4];

  // FFD8FFE1 start

  for(int i = 0; i < 4; i++)
  {
    c = getc(in);
    lb[i] = c;
    putc(c, out1);
  }

  if(((unsigned char)lb[0]) == 0xFF && ((unsigned char)lb[1]) == 0xD8 && ((unsigned char)lb[2]) == 0xFF && ((unsigned char)lb[3]) == 0xE1)
  {
    cerr << "Found start of first jpg file." << endl;
  }
  else
  {
    cerr << "File does not have valid start sequence." << endl;
    fclose(in);
    fclose(out1);
    fclose(out2);
    exit(1);
  }

  int status = 0;

  FILE * write = out1;

  while(c != EOF)
  {
    c = getc(in);
    if(status == 0 && ((unsigned char)c) == 0xFF)
    {
      lb[status] = c;
      status = 1;
    }
    else if(status == 1 && ((unsigned char)c) == 0xD8)
    {
      lb[status] = c;
      status = 2;
    }
    else if(status == 2 && ((unsigned char)c) == 0xFF)
    {
      lb[status] = c;
      status = 3;
    }
    else if(status == 3 && ((unsigned char)c) == 0xE1)
    {
      if(write == out2)
      {
        cerr << "Found too many file starts, output may not be valid." << endl;
      }
      lb[status] = c;
      status = 4;
      write = out2;
      cerr << "First file completed. Writing second file." << endl;
      for(int i = 0; i < status; i++)
      {
        putc(lb[i], write);
      }
      status = 0;
    }
    else
    {
      for(int i = 0; i < status; i++)
      {
        putc(lb[i], write);
      }
      status = 0;
      putc(c,write);
    }
  }

  fclose(out1);
  fclose(out2);
  fclose(in);

  cerr << "Successfully extracted stereo JPEG files." << endl;

  return 0;
}
